==> zither-1.1.16.tar.gz: zither_prj/sitar/scan_tcpport.c <==
#include<stdio.h>
#include<string.h>
#include<unistd.h>
#include<sys/socket.h>
#include<netinet/in_systm.h>
#include<netinet/in.h>
#include<netinet/ip.h>
#include<fcntl.h>
#include<arpa/inet.h>

#include<sys/stat.h>
#include<errno.h>

#include"sitar_scan.h"
#include"sitar_com.h"
#include<almemsys/almemsys.h>
#include"jpreturn.h"


/* 1-> port No' 2-> Ipsddr 3-> return data,, return status code 0=open or=close
int scan_tcp_connect(int, char *, char *, int);
*/


/*----------------- SCAN TCP CONNECT --------------------*/
char * scan_tcp_connect(int port, char * ip_addr, char * backdata, int timeout, int * stat)
{
char ipa2[IP_LEN];
char * ip_addr2 = ipa2;
int sk, re, retval, arglen;
unsigned long resoip;
struct sockaddr_in my_addr;
fd_set rfds;
struct timeval tv;

arglen = sizeof(int);

errno = 0;

if((sk = socket(AF_INET, SOCK_STREAM, 0))==-1){
   backdata=safe_sprintf(backdata,BUF_MAX,
    "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_1);
   * stat = -1;
   #ifdef TCP_BUG
   printf("TCP: scan_tcp_connect(1): NOT CLEATE SOCKET %s\n", backdata);
   #endif
   return backdata;
   }

my_addr.sin_family = AF_INET;
my_addr.sin_port = htons(port);

backdata = resolvename(backdata, ip_addr, timeout, &resoip);
my_addr.sin_addr.s_addr = resoip;

if(0 == (int)my_addr.sin_addr.s_addr || resoip == (int)NULL){
   backdata=safe_sprintf(backdata,BUF_MAX,"%s%s %d\n",
    SOCK_2, ip_addr, (int)my_addr.sin_addr.s_addr);
   #ifdef TCP_BUG
   printf("TCP: scan_tcp_connect(2): NOT RESOLV NAME %s\n", backdata);
   #endif
   close(sk);
   * stat = -1;
   return backdata;
   }

ip_addr2 = inet_ntoa(my_addr.sin_addr);

#ifdef TCP_BUG
printf("TCP: scan_tcp_connetc(3): start TCPCHECK host<%s> ip<%s>\n", ip_addr2, ip_addr);
#endif


// 非同期でTCPポートにゃEネクゃLッ@ッE
fcntl(sk, F_SETFL, O_NONBLOCK);
re = connect(sk, (struct sockaddr *)(long)&my_addr, sizeof(my_addr));


// 速攻ぃ@碃R立した堃G
if(re >= 0){
   backdata=safe_sprintf(backdata,BUF_MAX,
    "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_3);
   #ifdef TCP_BUG
   printf("TCP: scan_tcp_connect(4): CONNECT SUCCESS %s\n", backdata);
   #endif
   // 非同期通\Bから同期通\BぃH
   fcntl(sk, F_SETFL, 0);
   close(sk);
   * stat = 0;
   return backdata;
   }


// errnoが EINPROGRESS ぃ堃G合はselectぃ@待つ
if(errno == EINPROGRESS){
   tv.tv_sec = timeout;
   tv.tv_usec = 0;

   while(1){
      FD_ZERO(&rfds);
      FD_SET(sk, &rfds);
      retval = select(sk + 1, NULL, &rfds, NULL, &tv);
      // select ゃBッDッV
      if(retval < 0){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d %s %s", ip_addr, port, SOCK_15, SOCK_5);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connetc(5): select err BREAK-%d- %s\n", retval, backdata);
         #endif
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * stat = -1;
         return backdata;
         }
      // select time out ぃ@
      else if(retval == 0){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d %s %s", ip_addr, port, SOCK_16, SOCK_5);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connetc(6): select time out BREAK-%d- %s\n", retval, backdata);
         #endif
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * stat = -1;
         return backdata;
         }
      // select ぃLoopOut
      else{
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_3);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connect(7): select time succes BREAK-%d- %s\n", retval, backdata);
         #endif
         break;
         }
      }

   // 非同期通\Bから同期通\BぃH
   fcntl(sk, F_SETFL, 0);

   // ゃXゃAットぃH閁uするペンディッEめ|ゃBッDッVを戻
   if(getsockopt(sk, SOL_SOCKET, SO_ERROR, (void *)&retval, (void *)&arglen) < 0){
      backdata=safe_sprintf(backdata,BUF_MAX,
       "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_13);
      #ifdef TCP_BUG
      printf("TCP: scan_tcp_connect(8): getsockopt() Errors!\n");
      #endif
      close(sk);
      * stat = -1;
      return backdata;
      }
   else{
      // ゃBッDッVがある堃G
      if(retval){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d %s: TCP %s", ip_addr, port, (retval != 0) ? strerror(retval): "0", SOCK_5);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connect(9): 1 SO_ERROR %d -\n", retval);
         printf("%s\n", backdata);
         #endif
         close(sk);
         * stat = -1;
         return backdata;
         }
      // ゃBッDッVが無い堃G
      else{
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_3);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connect(10): -1 SO_ERROR %d -\n", retval);
         printf("%s\n", backdata);
         #endif
         close(sk);
         * stat = 0;
         return backdata;
         }
      }
   }

// ネットワッVゃbぃHIPやホゃPトが存在しない堃G合の凃
// 非同期通\Bから同期通\BぃH
fcntl(sk, F_SETFL, 0);

backdata=safe_sprintf(backdata,BUF_MAX,
 "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_14); 
#ifdef TCP_BUG
printf("TCP: scan_tcp_connect(11): --- LAST END ---\n");
printf("%s\n", backdata);
#endif

close(sk);
* stat = -1;
return backdata;

}

==> zither-1.1.18.tar.gz: zither_prj/sitar/scan_tcpport.c <==
#include<stdio.h>
#include<string.h>
#include<unistd.h>
#include<sys/socket.h>
#include<netinet/in_systm.h>
#include<netinet/in.h>
#include<netinet/ip.h>
#include<fcntl.h>
#include<arpa/inet.h>

#include<sys/stat.h>
#include<errno.h>

#include"sitar_scan.h"
#include"sitar_com.h"
#include<almemsys/almemsys.h>
#include"jpreturn.h"


/* 1-> port No' 2-> Ipsddr 3-> return data,, return status code 0=open or=close
int scan_tcp_connect(int, char *, char *, int);
*/


/*----------------- SCAN TCP CONNECT --------------------*/
char * scan_tcp_connect(int port, char * url_data, char * backdata, int timeout, int * stat)
{
char ipa2[IP_LEN];
char * ip_addr2 = ipa2;
char ip[BUF_URL];
char * ip_addr = ip;
char pas[BUF_URL];
char * url_pass = pas;
int sk, re, retval, arglen, port2;
unsigned long resoip;
struct sockaddr_in my_addr;
fd_set rfds;
struct timeval tv;

arglen = sizeof(int);

errno = 0;

memset(ip_addr, 0x00, BUF_URL);
memset(url_pass, 0x00, BUF_URL);

// portの設定よりもURLに含まれるポートを優先する
port2 = urldompass_port(url_data, ip_addr, url_pass, strlen(url_data));
if(port2 > 0){
   port = port2;
   }


if((sk = socket(AF_INET, SOCK_STREAM, 0))==-1){
   backdata=safe_sprintf(backdata,BUF_MAX,
    "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_1);
   * stat = -1;
   #ifdef TCP_BUG
   printf("TCP: scan_tcp_connect(1): NOT CLEATE SOCKET %s\n", backdata);
   #endif
   return backdata;
   }

my_addr.sin_family = AF_INET;
my_addr.sin_port = htons(port);

backdata = resolvename(backdata, ip_addr, timeout, &resoip);
my_addr.sin_addr.s_addr = resoip;

if(0 == (int)my_addr.sin_addr.s_addr || resoip == (int)NULL){
// backdata=safe_sprintf(backdata,BUF_MAX,"%s%s %d\n", SOCK_2, ip_addr, (int)my_addr.sin_addr.s_addr);
   backdata=safe_sprintf(backdata,BUF_MAX,"%s%s\n", SOCK_2, ip_addr);
   #ifdef TCP_BUG
   printf("TCP: scan_tcp_connect(2): NOT RESOLV NAME %s\n", backdata);
   #endif
   close(sk);
   * stat = -1;
   return backdata;
   }

ip_addr2 = inet_ntoa(my_addr.sin_addr);

#ifdef TCP_BUG
printf("TCP: scan_tcp_connetc(3): start TCPCHECK host<%s> ip<%s>\n", ip_addr2, ip_addr);
#endif

fcntl(sk, F_SETFL, O_NONBLOCK);
re = connect(sk, (struct sockaddr *)(long)&my_addr, sizeof(my_addr));

if(re >= 0){
   backdata=safe_sprintf(backdata,BUF_MAX,
    "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_3);
   #ifdef TCP_BUG
   printf("TCP: scan_tcp_connect(4): CONNECT SUCCESS %s\n", backdata);
   #endif
   fcntl(sk, F_SETFL, 0);
   close(sk);
   * stat = 0;
   return backdata;
   }

if(errno == EINPROGRESS){
   tv.tv_sec = timeout;
   tv.tv_usec = 0;

   while(1){
      FD_ZERO(&rfds);
      FD_SET(sk, &rfds);
      retval = select(sk + 1, NULL, &rfds, NULL, &tv);
      if(retval < 0){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d %s %s", ip_addr, port, SOCK_15, SOCK_5);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connetc(5): select err BREAK-%d- %s\n", retval, backdata);
         #endif
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * stat = -1;
         return backdata;
         }
      else if(retval == 0){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d %s %s", ip_addr, port, SOCK_16, SOCK_5);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connetc(6): select time out BREAK-%d- %s\n", retval, backdata);
         #endif
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * stat = -1;
         return backdata;
         }
      else{
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_3);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connect(7): select time succes BREAK-%d- %s\n", retval, backdata);
         #endif
         break;
         }
      }

   fcntl(sk, F_SETFL, 0);

   if(getsockopt(sk, SOL_SOCKET, SO_ERROR, (void *)&retval, (void *)&arglen) < 0){
      backdata=safe_sprintf(backdata,BUF_MAX,
       "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_13);
      #ifdef TCP_BUG
      printf("TCP: scan_tcp_connect(8): getsockopt() Errors!\n");
      #endif
      close(sk);
      * stat = -1;
      return backdata;
      }
   else{
      if(retval){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d %s: TCP %s", ip_addr, port, (retval != 0) ? strerror(retval): "0", SOCK_5);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connect(9): 1 SO_ERROR %d -\n", retval);
         printf("%s\n", backdata);
         #endif
         close(sk);
         * stat = -1;
         return backdata;
         }
      else{
         backdata=safe_sprintf(backdata,BUF_MAX,
          "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_3);
         #ifdef TCP_BUG
         printf("TCP: scan_tcp_connect(10): -1 SO_ERROR %d -\n", retval);
         printf("%s\n", backdata);
         #endif
         close(sk);
         * stat = 0;
         return backdata;
         }
      }
   }

fcntl(sk, F_SETFL, 0);

backdata=safe_sprintf(backdata,BUF_MAX,
 "IP=%s Port=%d TCP %s", ip_addr, port, SOCK_14); 
#ifdef TCP_BUG
printf("TCP: scan_tcp_connect(11): --- LAST END ---\n");
printf("%s\n", backdata);
#endif

close(sk);
* stat = -1;
return backdata;

}
